#!/usr/bin/python
# -*- coding: utf-8 -*-

"""Module with tests for wirelesstag platform."""

import unittest

import test.mock as MOCK
import requests_mock


import wirelesstagpy
import wirelesstagpy.constants as CONST
from wirelesstagpy.sensortag import SensorTag
from wirelesstagpy.notificationconfig import NotificationConfig

USERNAME = 'foobar'
PASSWORD = 'deadbeef'


class TestWirelessTags(unittest.TestCase):
    """Tests for WirelessTags."""

    def setUp(self):
        """Set up wirelesstags platform module."""
        self.platform_no_cred = wirelesstagpy.WirelessTags(username='', password='')
        self.platform = wirelesstagpy.WirelessTags(username=USERNAME, password=PASSWORD)

    def tearDown(self):
        """Clean up after each test."""
        self.platform = None
        self.platform_no_cred = None

    @requests_mock.mock()
    def tests_full_data_load(self, m):
        """Test full data logic."""
        m.post(CONST.SIGN_IN_URL, text=MOCK.LOGIN_RESPONSE)
        m.post(CONST.IS_SIGNED_IN_URL, text=MOCK.LOGIN_RESPONSE)
        m.post(CONST.GET_TAGS_URL, text=MOCK.TAGS_LIST_RESPONSE)

        tags = self.platform.load_tags()
        self.assertTrue(list(tags.keys()))
        self.assertTrue(self.platform.use_celsius)

        for uuid, sensor in tags.items():
            self.assertEqual(uuid, sensor.uuid)

            self.assertIsNotNone(sensor.name)
            self.assertIsNotNone(sensor.uuid)
            self.assertIsNotNone(sensor.tag_manager_mac)

            self.assertIsNotNone(sensor.temperature)
            self.assertTrue(isinstance(sensor.temperature, float))

            self.assertIsNotNone(sensor.humidity)
            self.assertTrue(isinstance(sensor.humidity, float))

            self.assertIsNotNone(sensor.battery_remaining)

            self.assertIsNotNone(sensor.battery_volts)
            self.assertIsNotNone(sensor.tag_type)
            self.assertIsNotNone(sensor.comment)
            self.assertIsNotNone(sensor.is_alive)
            self.assertIsNotNone(sensor.signal_strength)
            self.assertIsNotNone(sensor.beep_option)
            self.assertIsNotNone(sensor.is_in_range)
            self.assertIsNotNone(sensor.hw_revision)
            self.assertIsNotNone(sensor.sw_version)
            self.assertIsNotNone(sensor.power_consumption)

            self.assertFalse(sensor.is_motion_sensor_armed)
            self.assertFalse(sensor.is_humidity_sensor_armed)
            self.assertFalse(sensor.is_temperature_sensor_armed)
            self.assertFalse(sensor.is_moisture_sensor_armed)
            self.assertTrue(sensor.moisture_sensor_state == 0)
            self.assertFalse(sensor.is_light_sensor_armed)
            print("tag: {} last updated: {}".format(sensor, sensor.time_since_last_update))

    def test_alspro_tag_binary_states(self):
        """Test avaiable binary states for als pro tag."""
        tag = SensorTag(MOCK.ALS_PRO, self.platform, '0d0d0d0d0d0d')
        self.assertTrue(tag.is_moved)
        self.assertFalse(tag.is_door_open)
        self.assertTrue(tag.is_cold)
        self.assertFalse(tag.is_heat)
        self.assertFalse(tag.is_too_dry)
        self.assertFalse(tag.is_too_humid)
        self.assertFalse(tag.is_leaking)
        self.assertTrue(tag.is_light_on)
        self.assertFalse(tag.is_battery_low)
        self.assertIsNotNone(str(tag))
        self.assertIsNotNone(tag.tag_manager_mac)

    def test_water_tag_binary_states(self):
        """Test avaiable binary states for als pro tag."""
        tag = SensorTag(MOCK.WATERSENSOR, self.platform, '0d0d0d0d0d0d')
        self.assertFalse(tag.is_moved)
        self.assertFalse(tag.is_door_open)
        self.assertFalse(tag.is_cold)
        self.assertFalse(tag.is_heat)
        self.assertFalse(tag.is_too_dry)
        self.assertFalse(tag.is_too_humid)
        self.assertTrue(tag.is_leaking)
        self.assertFalse(tag.is_light_on)
        self.assertFalse(tag.is_battery_low)
        self.assertIsNotNone(str(tag))
        self.assertIsNotNone(tag.tag_manager_mac)

    def test_13bit_tag_binary_states(self):
        """Test avaiable binary states for als pro tag."""
        tag = SensorTag(MOCK.BITS13, self.platform, '0d0d0d0d0d0d')
        self.assertFalse(tag.is_moved)
        self.assertTrue(tag.is_door_open)
        self.assertFalse(tag.is_cold)
        self.assertTrue(tag.is_heat)
        self.assertFalse(tag.is_too_dry)
        self.assertTrue(tag.is_too_humid)
        self.assertFalse(tag.is_leaking)
        self.assertFalse(tag.is_light_on)
        self.assertTrue(tag.is_battery_low)
        self.assertIsNotNone(str(tag))
        self.assertIsNotNone(tag.tag_manager_mac)

    @requests_mock.mock()
    def test_failed_login(self, m):
        """Verify handling of incorrect credentials."""
        m.post(CONST.SIGN_IN_URL, status_code=500)
        with self.assertRaises(wirelesstagpy.WirelessTagsException):
            tags = self.platform_no_cred.load_tags()
            print('tags: {}'.format(tags))

    @requests_mock.mock()
    def test_failed_load_tags(self, m):
        """Verify exception handling while server is not responding."""
        m.post(CONST.SIGN_IN_URL, text=MOCK.LOGIN_RESPONSE)
        m.post(CONST.IS_SIGNED_IN_URL, text=MOCK.LOGIN_RESPONSE)
        m.post(CONST.GET_TAGS_URL, text='{no really valid payload}', status_code=500)

        tags = self.platform.load_tags()
        self.assertFalse(tags.keys())

    @requests_mock.mock()
    def test_check_of_sign_in_status(self, m):
        """Verify exception handling while unable to get respnse on signin status."""
        m.post(CONST.SIGN_IN_URL, text=MOCK.LOGIN_RESPONSE)
        m.post(CONST.IS_SIGNED_IN_URL, text='{no really valid payload}', status_code=500)
        m.post(CONST.GET_TAGS_URL, text=MOCK.TAGS_LIST_RESPONSE)

        tags = self.platform.load_tags()
        description = str(self.platform)
        self.assertTrue(description.startswith('WirelessTagsPlatform: using celsius'))
        print('platform: {}'.format(description))
        self.assertTrue(tags.keys())

    @requests_mock.mock()
    def test_arm_disarm_motion(self, m):
        """Verify arm/disarm basic behaviour."""
        m.post(CONST.SIGN_IN_URL, text=MOCK.LOGIN_RESPONSE)
        m.post(CONST.IS_SIGNED_IN_URL, text=MOCK.LOGIN_RESPONSE)
        m.post(CONST.ARM_MOTION_URL, text=MOCK.ARM_MOTION_RESPONSE)
        m.post(CONST.DISARM_MOTION_URL, text=MOCK.DISARM_MOTION_RESPONSE)

        sensor = self.platform.arm_motion(1)
        self.assertEqual(sensor.tag_id, 1)
        self.assertTrue(sensor.is_motion_sensor_armed)

        sensor = self.platform.disarm_motion(1)
        self.assertEqual(sensor.tag_id, 1)
        self.assertFalse(sensor.is_motion_sensor_armed)

    @requests_mock.mock()
    def test_arm_disarm_motion_with_mac(self, m):
        """Verify arm/disarm basic behaviour."""
        m.post(CONST.SIGN_IN_URL, text=MOCK.LOGIN_RESPONSE)
        m.post(CONST.IS_SIGNED_IN_URL, text=MOCK.LOGIN_RESPONSE)
        m.post(CONST.ARM_MOTION_URL, text=MOCK.ARM_MOTION_RESPONSE)
        m.post(CONST.DISARM_MOTION_URL, text=MOCK.DISARM_MOTION_RESPONSE)

        sensor = self.platform.arm_motion(1, '0d0d0d0d0d0d')
        self.assertEqual(sensor.tag_id, 1)
        self.assertEqual(sensor.tag_manager_mac, '0d0d0d0d0d0d')
        self.assertTrue(sensor.is_motion_sensor_armed)

        sensor = self.platform.disarm_motion(1, '0d0d0d0d0d0d')
        self.assertEqual(sensor.tag_id, 1)
        self.assertEqual(sensor.tag_manager_mac, '0d0d0d0d0d0d')
        self.assertFalse(sensor.is_motion_sensor_armed)

    @requests_mock.mock()
    def test_arm_disarm_temperature(self, m):
        """Verify arm/disarm temperature monitoring."""
        m.post(CONST.SIGN_IN_URL, text=MOCK.LOGIN_RESPONSE)
        m.post(CONST.IS_SIGNED_IN_URL, text=MOCK.LOGIN_RESPONSE)
        m.post(CONST.ARM_TEMPERATURE_URL, text=MOCK.ARM_TEMP_RESPONSE)
        m.post(CONST.DISARM_TEMPERATURE_URL, text=MOCK.DISARM_TEMP_RESPONSE)

        sensor = self.platform.arm_temperature(1)
        self.assertEqual(sensor.tag_id, 1)
        self.assertTrue(sensor.is_temperature_sensor_armed)

        sensor = self.platform.disarm_temperature(1)
        self.assertEqual(sensor.tag_id, 1)
        self.assertFalse(sensor.is_temperature_sensor_armed)

    @requests_mock.mock()
    def test_arm_disarm_humidity(self, m):
        """Verify arm/disarm temperature monitoring."""
        m.post(CONST.SIGN_IN_URL, text=MOCK.LOGIN_RESPONSE)
        m.post(CONST.IS_SIGNED_IN_URL, text=MOCK.LOGIN_RESPONSE)
        m.post(CONST.ARM_HUMIDITY_URL, text=MOCK.ARM_HUMIDITY_RESPONSE)
        m.post(CONST.DISARM_HUMIDITY_URL, text=MOCK.DISARM_HUMIDITY_RESPONSE)

        sensor = self.platform.arm_humidity(1)
        self.assertEqual(sensor.tag_id, 1)
        self.assertTrue(sensor.is_humidity_sensor_armed)

        sensor = self.platform.disarm_humidity(1)
        self.assertEqual(sensor.tag_id, 1)
        self.assertFalse(sensor.is_humidity_sensor_armed)

    @requests_mock.mock()
    def test_arm_disarm_light(self, m):
        """Verify arm/disarm temperature monitoring."""
        m.post(CONST.SIGN_IN_URL, text=MOCK.LOGIN_RESPONSE)
        m.post(CONST.IS_SIGNED_IN_URL, text=MOCK.LOGIN_RESPONSE)
        m.post(CONST.ARM_LIGHT_URL, text=MOCK.ARM_LIGHT_RESPONSE)
        m.post(CONST.DISARM_LIGHT_URL, text=MOCK.DISARM_LIGHT_RESPONSE)

        sensor = self.platform.arm_light(1)
        self.assertEqual(sensor.tag_id, 1)
        self.assertTrue(sensor.is_light_sensor_armed)

        sensor = self.platform.disarm_light(1)
        self.assertEqual(sensor.tag_id, 1)
        self.assertFalse(sensor.is_light_sensor_armed)

    @requests_mock.mock()
    def test_arm_disarm_failed(self, m):
        """Verify arm/disarm temperature monitoring."""
        m.post(CONST.SIGN_IN_URL, text=MOCK.LOGIN_RESPONSE)
        m.post(CONST.IS_SIGNED_IN_URL, text=MOCK.LOGIN_RESPONSE)
        m.post(CONST.ARM_LIGHT_URL, text='{no really valid payload}')

        sensor = self.platform.arm_light(1)
        self.assertIsNone(sensor)

    @requests_mock.mock()
    def test_fetch_push_notifications(self, m):
        """Test fetch installed push notifications."""
        m.post(CONST.SIGN_IN_URL, text=MOCK.LOGIN_RESPONSE)
        m.post(CONST.IS_SIGNED_IN_URL, text=MOCK.LOGIN_RESPONSE)
        m.post(CONST.LOAD_EVENT_URL_CONFIG_URL, text=MOCK.LOAD_EVENT_URL_CONFIG_RESPONSE)

        notifications = self.platform.fetch_push_notifications(1)
        self.assertEqual(len(notifications), 22)

        enabled_notification = list(filter(lambda x: x.is_enabled, notifications))
        self.assertEqual(len(enabled_notification), 1)

        config = enabled_notification[0]
        self.assertEqual(config.name, 'update')
        self.assertEqual(config.is_local, True)
        self.assertEqual(config.verb, 'POST')
        self.assertEqual(config.content, 'message')
        self.assertEqual(config.url, 'http://host_name/update')

    @requests_mock.mock()
    def test_fail_fetch_notifications(self, m):
        """Test for catching errors logic inside fetch method."""
        m.post(CONST.SIGN_IN_URL, text=MOCK.LOGIN_RESPONSE)
        m.post(CONST.IS_SIGNED_IN_URL, text=MOCK.LOGIN_RESPONSE)
        m.post(CONST.LOAD_EVENT_URL_CONFIG_URL, text='kinda failed', status_code=500)

        notifications = self.platform.fetch_push_notifications(1)
        self.assertEqual(len(notifications), 0)

    def test_notification(self):
        """Test NotificationConfig properties."""
        config = NotificationConfig('update', MOCK.UPDATE_NOTIFICATION_CONFIG)

        self.assertEqual(config.name, 'update')
        self.assertEqual(config.is_local, True)
        self.assertEqual(config.verb, 'POST')
        self.assertEqual(config.content, "{\"name\":\"{0}\",\"id\":{1},\"temp\": {2}, \"cap\":{3},\"lux\":{4}}")
        self.assertEqual(config.url, 'http://10.10.0.2/api/events/update_tags')
        self.assertIsNotNone(str(config))

    @requests_mock.mock()
    def test_install_notifications(self, m):
        """Test install push notifications."""
        m.post(CONST.SIGN_IN_URL, text=MOCK.LOGIN_RESPONSE)
        m.post(CONST.IS_SIGNED_IN_URL, text=MOCK.LOGIN_RESPONSE)
        m.post(CONST.LOAD_EVENT_URL_CONFIG_URL, text=MOCK.LOAD_EVENT_URL_CONFIG_RESPONSE)
        m.post(CONST.SAVE_EVENT_URL_CONFIG_URL, text=MOCK.LOAD_EVENT_URL_CONFIG_RESPONSE)

        notifications = [
            NotificationConfig("update", MOCK.UPDATE_NOTIFICATION_CONFIG)
        ]

        result = self.platform.install_push_notification(1, notifications, False)
        self.assertTrue(result)
        print('succeed: ', result)

    @requests_mock.mock()
    def test_fail_setup_notifications(self, m):
        """Test for catching errors logic inside install notifications method."""
        m.post(CONST.SIGN_IN_URL, text=MOCK.LOGIN_RESPONSE)
        m.post(CONST.IS_SIGNED_IN_URL, text=MOCK.LOGIN_RESPONSE)
        m.post(CONST.LOAD_EVENT_URL_CONFIG_URL, text='kinda failed', status_code=500)

        notifications = [
            NotificationConfig("update", MOCK.UPDATE_NOTIFICATION_CONFIG)
        ]
        result = self.platform.install_push_notification(1, notifications, False)
        self.assertFalse(result)

    def test_leak_binary_event(self):
        """Test water leak binary events logic."""
        mac = '0d0d0d0d0d0d'
        tag = SensorTag(MOCK.WATERSENSOR, self.platform, mac)

        event = tag.event[CONST.EVENT_WET]
        self.assertIsNone(event)

        event = tag.event[CONST.EVENT_MOISTURE]
        self.assertIsNotNone(event)
        self.assertTrue(event.is_state_on)

        url = 'http://10.10.0.2/api/events/update_tags'
        notifications = event.build_notifications(url, mac)
        self.assertTrue(len(notifications) == 2)

        config = notifications[0]
        self.assertTrue(config.name in ['water_detected', 'water_dried'])
        self.assertTrue(config.is_enabled)
        self.assertTrue(config.is_local)
        self.assertEqual(config.url, url)
        self.assertEqual(config.verb, 'POST')

    def test_motion_binary_event(self):
        """Test motion binary event for PIR sensor."""
        mac = '12'
        tag = SensorTag(MOCK.PIRSENSOR, self.platform, mac)
        event = tag.event[CONST.EVENT_MOTION]
        same_event = tag.event[CONST.EVENT_MOTION]
        self.assertTrue(event == same_event)
        self.assertIsNotNone(event)
        self.assertFalse(event.is_state_on)

        url = 'http://10.10.0.2/api/events/update_tags'
        notifications = event.build_notifications(url, mac)
        self.assertTrue(len(notifications) == 1)

        config = notifications[0]
        self.assertEqual(config.name, 'motion_detected')
        self.assertTrue(config.is_enabled)
        self.assertTrue(config.is_local)
        self.assertEqual(config.url, url)
        self.assertEqual(config.verb, 'POST')

    def test_make_update_event(self):
        """Test logic to create update event push notification."""
        url = 'https://update/tag'
        mac = '12'
        config = NotificationConfig.make_config_for_update_event(url, mac)
        self.assertIsNotNone(config)
        self.assertEqual(config.url, url)
        self.assertEqual(config.verb, 'POST')
        self.assertTrue(config.is_enabled)
        self.assertTrue(config.is_local)

    def test_sensors(self):
        """Test sensors supported by tag."""
        mac = '12'
        tag = SensorTag(MOCK.PIRSENSOR, self.platform, mac)
        sensor = tag.sensor[CONST.SENSOR_TEMPERATURE]
        same_sensor = tag.sensor[CONST.SENSOR_TEMPERATURE]
        self.assertIsNotNone(sensor)
        self.assertTrue(sensor == same_sensor)
        self.assertEqual(sensor.value, 23.924919128417969)
        self.assertIsNotNone(str(sensor))

        event = {'name': 'Kitchen', 'id': 8,
                 'temp': 24.1517391204834, 'cap': 0,
                 'lux': 0, 'mac': '12'}
        value = sensor.value_from_update_event(event)
        self.assertEqual(value, 24.1517391204834)

        value = sensor.value_from_update_event({'name': 'no-exist'})
        self.assertEqual(value, 0)

        no_sensor = tag.sensor[CONST.SENSOR_LIGHT]
        self.assertIsNone(no_sensor)

    def test_switches(self):
        """Test allowed monitoring entities."""
        mac = '12'
        tag = SensorTag(MOCK.PIRSENSOR, self.platform, mac)
        switch_types = tag.allowed_monitoring_types
        self.assertIn(CONST.ARM_TEMPERATURE, switch_types)
        self.assertIn(CONST.ARM_HUMIDITY, switch_types)
        self.assertIn(CONST.ARM_MOTION, switch_types)

    def test_no_sensor_for_subscription(self):
        """Test for non-existing attribute."""
        mac = '12'
        tag = SensorTag(MOCK.PIRSENSOR, self.platform, mac)
        with self.assertRaises(AttributeError):
            no_message = tag.message
            print('message: {}'.format(no_message))



#!/usr/bin/python
# -*- coding: utf-8 -*-

"""
Model class event with binary state (on/off) for wirelesstags platform.

Copyrights: (c) 2018 Sergiy Maysak, see LICENSE file for details
Creation Date: 8/20/2018
"""

import weakref
import wirelesstagpy.constants as CONST
import wirelesstagpy.notificationconfig as NS

# Binary event type specification:
# Human Readable Name,
# device_class/type,
# push notification type representing 'on',
# push notification type representing 'of',
# attr to check state
BINARY_EVENT_SPECS = {
    CONST.EVENT_PRESENCE: ['Presence', CONST.EVENT_PRESENCE, 'is_in_range', {
        "on": "oor",
        "off": "back_in_range"
    }, 2],
    CONST.EVENT_MOTION: ['Motion', CONST.EVENT_MOTION, 'is_moved', {
        "on": "motion_detected",
    }, 5],
    CONST.EVENT_DOOR: ['Door', CONST.EVENT_DOOR, 'is_door_open', {
        "on": "door_opened",
        "off": "door_closed"
    }, 5],
    CONST.EVENT_COLD: ['Cold', CONST.EVENT_COLD, 'is_cold', {
        "on": "temp_toolow",
        "off": "temp_normal"
    }, 4],
    CONST.EVENT_HEAT: ['Heat', CONST.EVENT_HEAT, 'is_heat', {
        "on": "temp_toohigh",
        "off": "temp_normal"
    }, 4],
    CONST.EVENT_DRY: ['Too dry', CONST.EVENT_DRY, 'is_too_dry', {
        "on": "too_dry",
        "off": "cap_normal"
    }, 2],
    CONST.EVENT_WET: ['Too wet', CONST.EVENT_WET, 'is_too_humid', {
        "on": "too_humid",
        "off": "cap_normal"
    }, 2],
    CONST.EVENT_LIGHT: ['Light', CONST.EVENT_LIGHT, 'is_light_on', {
        "on": "too_bright",
        "off": "light_normal"
    }, 1],
    CONST.EVENT_MOISTURE: ['Leak', CONST.EVENT_MOISTURE, 'is_leaking', {
        "on": "water_detected",
        "off": "water_dried",
    }, 1],
    CONST.EVENT_BATTERY: ['Low Battery', CONST.EVENT_BATTERY, 'is_battery_low', {
        "on": "low_battery"
    }, 3]
}


class BinaryEvent:
    """Model representing event with binary state."""

    @classmethod
    def make_event(cls, event_type, sensortag):
        """Create event of specific type and tag."""
        spec_list = BINARY_EVENT_SPECS[event_type]
        return cls(spec_list, sensortag)

    def __init__(self, spec_list, sensortag):
        """Init event with attributes list and tag."""
        self.human_readable_name = spec_list[0]
        self.type = spec_list[1]
        self._tag_attr = spec_list[2]
        self.notification_spec = spec_list[3]
        self.tag_id_template = spec_list[4]
        self._parent_tag = weakref.ref(sensortag)

        if (sensortag.tag_type == CONST.WIRELESSTAG_TYPE_PIR and
                self.type == CONST.EVENT_MOTION):
            self.tag_id_template = 2

    def build_notifications(self, url, mac):
        """Construct push notifications for event."""
        configs = []
        for state, name in self.notification_spec.items():
            content = ('{"type": "' + self.type +
                       '", "id":{' + str(self.tag_id_template) +
                       '}, "mac":\"' + mac +
                       '\", "state": \"' + state + '\"}')
            config = NS.NotificationConfig.make_post_local(name, url, content)
            configs.append(config)

        return configs

    @property
    def is_state_on(self):
        """Return binary event state."""
        return getattr(self._parent_tag(), self._tag_attr, False)


class BinaryEventProxy:
    """Proxy model to handle subscripting for single binry event."""

    def __init__(self, tag):
        """Init proxy with parent tag."""
        self._parent_tag = weakref.ref(tag)

    def __getitem__(self, key):
        """Return binary event instance for key."""
        return self._parent_tag().event_for_type(key)



#!/usr/bin/python
# -*- coding: utf-8 -*-

"""
Simple python library for wirelesstags REST API.

See http://wirelesstag.net/apidoc.html for more details.
Current implementation includes:
- loading info for all available to user sensor tags
- manage local push notifications
- arm/disarm tags on monitoring for
humidity, temp, light and motion
mytaglist.com account credentials are needed to use this lib.
Enabling tags sharing is not required.

"Wireless Sensor Tags", "KumoSensor" and "Kumostat" are
trademarks of Cao Gadgets LLC,
see www.wirelesstag.net for more information.
I am in no way affiliated with Cao Gadgets LLC.

Copyrights: (c) 2018 Sergiy Maysak, see LICENSE file for details
Creation Date: 3/7/2018.
"""

import json
import time
from datetime import datetime
import logging
import requests

from wirelesstagpy.sensortag import SensorTag
from wirelesstagpy.exceptions import WirelessTagsException
from wirelesstagpy.notificationconfig import NotificationConfig
import wirelesstagpy.constants as CONST

_LOGGER = logging.getLogger(__name__)


class WirelessTags:
    """Principal class for Wireless Sensors Tags."""

    _HEADERS = CONST.HEADERS
    _SIGN_IN_URL = CONST.SIGN_IN_URL
    _IS_SIGNED_IN_URL = CONST.IS_SIGNED_IN_URL
    _GET_TAGS_URL = CONST.GET_TAGS_URL

    def __init__(self, username, password):
        """Initialize wirelesstags platform."""
        self._username = username
        self._password = password
        self._cookies = None
        self._tags = {}

        # time interval from last reload
        self._last_load_time = 0

        # in seconds
        self._postback_interval = 600

        self.use_celsius = True

        # server time in utc (filetime format)
        self._server_time = 0

        # array of tag managers mac addresses
        self.mac_addresses = []

    def load_tags(self):
        """Load all registered tags."""
        if self._needs_reload:
            cookies = self._auth_cookies
            try:
                response = requests.post(
                    self._GET_TAGS_URL, headers=self._HEADERS, cookies=cookies)

                # remember time of load for cache/reload management
                self._last_load_time = time.time()

                json_tags_spec = response.json()
                tags = json_tags_spec['d']
                for tag in tags:
                    uuid = tag['uuid']
                    # save mac - a unique identifier of specific tag manager
                    mac = tag['mac'] if 'mac' in tag else None
                    self._tags[uuid] = SensorTag(tag, self, mac)
                    self._register_mac(mac)
                _LOGGER.info("Tags reloaded at: %s", datetime.now())
            except Exception as error:
                _LOGGER.error("failed to load tags - %s", error)

        return self._tags

    def arm_motion(self, tag_id, mac=None):
        """Arm motion sensor to monitor changes."""
        payload = {"id": tag_id, "door_mode_set_closed": True}
        return self._arm_control_tag(tag_id, CONST.ARM_MOTION_URL,
                                     mac, payload)

    def arm_temperature(self, tag_id, mac=None):
        """Arm temperature sensor to monitor changes."""
        return self._arm_control_tag(tag_id, CONST.ARM_TEMPERATURE_URL, mac)

    def arm_humidity(self, tag_id, mac=None):
        """Arm humidity sensor to monitor changes."""
        return self._arm_control_tag(tag_id, CONST.ARM_HUMIDITY_URL, mac)

    def arm_light(self, tag_id, mac=None):
        """Arm light sensor to monitor changes."""
        return self._arm_control_tag(tag_id, CONST.ARM_LIGHT_URL, mac)

    def disarm_motion(self, tag_id, mac=None):
        """Disarm motion sensor to monitor changes."""
        return self._arm_control_tag(tag_id, CONST.DISARM_MOTION_URL, mac)

    def disarm_temperature(self, tag_id, mac=None):
        """Disarm temperature sensor to monitor changes."""
        return self._arm_control_tag(tag_id, CONST.DISARM_TEMPERATURE_URL, mac)

    def disarm_humidity(self, tag_id, mac=None):
        """Disarm humidity sensor to monitor changes."""
        return self._arm_control_tag(tag_id, CONST.DISARM_HUMIDITY_URL, mac)

    def disarm_light(self, tag_id, mac=None):
        """Arm light sensor to monitor changes."""
        return self._arm_control_tag(tag_id, CONST.DISARM_LIGHT_URL, mac)

    def install_push_notification(
            self, tag_id, notifications, apply_to_all=False, tag_manager_mac=None):
        """Install set of push notifications for specified tag."""
        def list_to_spec(array):
            """Sub-func to represent notifications as dictionary."""
            spec = {}
            for item in array:
                spec[item.name] = item.spec
            return spec

        installed_spec = list_to_spec(self.fetch_push_notifications(tag_id))
        requested_spec = list_to_spec(notifications)

        new_config = installed_spec
        for name, _ in installed_spec.items():
            if name in requested_spec:
                new_config[name] = requested_spec[name]

        succeed = True
        cookies = self._auth_cookies
        try:
            payload = {
                "config": new_config,
                "applyAll": apply_to_all,
                "id": tag_id if not apply_to_all else -1
            }
            headers = self._headers_for_mac(tag_manager_mac)
            response = requests.post(CONST.SAVE_EVENT_URL_CONFIG_URL,
                                     headers=headers,
                                     cookies=cookies,
                                     data=json.dumps(payload))
            succeed = "d" in response.json()
        except Exception as error:
            _LOGGER.error("Failed to save notifications config: %s - %s",
                          tag_id, error)
            succeed = False

        return succeed

    def fetch_push_notifications(self, tag_id, tag_manager_mac=None):
        """Read from tags manager current set of push notifications."""
        cookies = self._auth_cookies
        notifications = []
        try:
            payload = json.dumps({"id": tag_id})
            headers = self._headers_for_mac(tag_manager_mac)
            response = requests.post(
                CONST.LOAD_EVENT_URL_CONFIG_URL, headers=headers,
                cookies=cookies, data=payload)
            json_notifications_spec = response.json()
            set_spec = json_notifications_spec['d']
            for name, spec in set_spec.items():
                if "url" in spec:
                    notifications.append(NotificationConfig(name, spec))
        except Exception as error:
            _LOGGER.error("failed to fetch : %s - %s", tag_id, error)
        return notifications

    def _arm_control_tag(self, tag_id, url, tag_manager_mac=None, own_payload=None):
        """Arm sensor with specified id and url to monitor changes."""
        cookies = self._auth_cookies
        sensor_tag = None
        try:
            payload = json.dumps((
                {"id": tag_id} if own_payload is None else own_payload))
            headers = self._headers_for_mac(tag_manager_mac)
            response = requests.post(
                url, headers=headers, cookies=cookies, data=payload)
            json_tags_spec = response.json()
            tag = json_tags_spec['d']
            uuid = tag['uuid']
            self._tags[uuid] = SensorTag(tag, self, tag_manager_mac)
            sensor_tag = self._tags[uuid]
        except Exception as error:
            _LOGGER.error("failed to arm/disarm for tag id: %s - %s",
                          tag_id, error)
        return sensor_tag

    def _headers_for_mac(self, tag_manager_mac):
        """Build headers for http request."""
        headers = self._HEADERS
        # combination of tag_id and X-Set-Mac header with mac
        # allows to uniquely identify tag across multiple tag managers
        if tag_manager_mac is not None:
            headers['X-Set-Mac'] = tag_manager_mac
        return headers

    def _login(self):
        """Perform user login."""
        auth = json.dumps({
            "email": self._username, "password": self._password})
        try:
            response = requests.post(
                self._SIGN_IN_URL, headers=self._HEADERS, data=auth)
            json_response = response.json()

            self._update_server_settings(json_response['d'])
            self._cookies = response.cookies
        except Exception as error:
            _LOGGER.debug("Failed to login to %s - %s", CONST.BASEURL, error)
            self._cookies = None
            raise WirelessTagsException("Unable to login to wirelesstags.net"
                                        " - check your credentials")

        _LOGGER.info("Login successful")

        return self._cookies

    def _update_server_settings(self, server_info):
        self._postback_interval = server_info['postbackInterval']
        self._server_time = server_info['serverTime']
        self.use_celsius = (server_info['temp_unit'] == 0)

    @property
    def _is_signed_in(self):
        cookies = self._cookies
        result = False
        if cookies is None:
            cookies = self._login()
        try:
            response = requests.post(
                self._IS_SIGNED_IN_URL, headers=self._HEADERS, cookies=cookies)
            json_response = response.json()
            if 'd' in json_response:
                self._update_server_settings(json_response['d'])
                result = True
        except Exception as error:
            _LOGGER.debug("Failed to check signin status - %s", error)
            self._cookies = None
        return result

    @property
    def _auth_cookies(self):
        if self._is_signed_in:
            return self._cookies

        return self._login()

    @property
    def _needs_reload(self):
        elapsed = time.time() - self._last_load_time
        return elapsed > self._postback_interval

    def _register_mac(self, mac):
        if mac not in self.mac_addresses:
            self.mac_addresses.append(mac)

    def __str__(self):
        """Return string representation of wirelesstags platform."""
        temperature_str = 'celsius' if self.use_celsius else 'fahrenheit'
        return "WirelessTagsPlatform: using {}," \
               "update interval: {} server time: {} tags: {}"\
               .format(temperature_str, self._postback_interval,
                       self._server_time, self._tags)



#!/usr/bin/python
# -*- coding: utf-8 -*-

"""
Defines sensor model for wirelesstags platform.

Tags could of different types with different set of active working attributes.

Copyrights: (c) 2018 Sergiy Maysak, see LICENSE file for details
Creation Date: 3/7/2018
"""

import logging
from datetime import datetime

import wirelesstagpy.utils as UTILS
import wirelesstagpy.constants as CONST
from wirelesstagpy.binaryevent import (
    BinaryEvent,
    BinaryEventProxy,
    BINARY_EVENT_SPECS)
from wirelesstagpy.sensor import (
    Sensor,
    SensorProxy)

_LOGGER = logging.getLogger(__name__)


class SensorTag:
    """Model representing single wireless sensor tag."""

    def __init__(self, info, platform, mac=None):
        """Init with dictionary and parent WirelessTagPlatform."""
        self._info = info
        self.uuid = self._info['uuid']
        self.tag_id = self._info['slaveId']
        self.name = self._info['name']
        self.temperature = self._info['temperature']
        self.humidity = self._info['cap']
        self.moisture = self.humidity

        # ambient light - working range: 0.1 lux to 200,000 lux
        self.light = self._info['lux']

        # mac address of tag manager this instance belong to
        self.tag_manager_mac = mac

        # binary events specs - built in lazy fashion
        # key - binary event type, value - instance of BinaryEvent
        self._event_specs = {}

        # spec sensors entities
        # key - sensor type, value - instance of Sensor
        self._sensors_spec = {}

    @property
    def battery_remaining(self):
        """Return float in percents of remaining battery charge."""
        return self._info['batteryRemaining']

    @property
    def battery_volts(self):
        """Return current voltage of battery as float in volts."""
        return self._info['batteryVolt']

    @property
    def low_battery_threshold(self):
        """Return amount of volts triggering low battery notification."""
        return self._info['LBTh']

    @property
    def tag_type(self):
        """Int representing tag type."""
        return self._info['tagType']

    @property
    def comment(self):
        """Return comment assigned by end user for this instance."""
        return self._info['comment']

    @property
    def is_alive(self):
        """Bool indicating if hw instance is alive."""
        return self._info['alive']

    @property
    def signal_strength(self):
        """Int with signal strength."""
        return self._info['signaldBm']

    @property
    def beep_option(self):
        """Enum value indicating selected beep option (5, 10, 15 times etc)."""
        return self._info['beepDurationDefault']

    @property
    def is_in_range(self):
        """Is in range (bool).

        Indicates if instance is in range of tags bridge and
        can be accessible for data (as bool).
        """
        return self._info['OutOfRange'] is False

    @property
    def hw_revision(self):
        """Hardware revision - as int."""
        return self._info['rev']

    @property
    def sw_version(self):
        """Software/Firmware revision - as int."""
        return self._info['version1']

    @property
    def power_consumption(self):
        """Power consumption in percents from max possible."""
        level = self._info['txpwr']
        return level * 100.0 / 255.0

    @property
    def last_load_time(self):
        """Datetime in utc. Represents last time when data from sensors was received."""
        communication_time = self._info['lastComm']
        return UTILS.convert_filetime_to_dt(communication_time)

    @property
    def time_since_last_update(self):
        """Time delta since last sensor data load."""
        return datetime.utcnow() - self.last_load_time

    @property
    def motion_state(self):
        """Return state of motion sensor."""
        # spec = {0: 'Disarmed', 1: 'Armed', 2: 'Moved', 3: 'Opened', 4: 'Closed',
        #        5: 'DetectedMovement', 6: 'TimedOut', 7: 'Stabilizing', 8: 'CarriedAway',
        #        9: 'InFreeFall'}
        return self._info['eventState']

    @property
    def is_motion_sensor_armed(self):
        """Return True if tag armed and listens to motion evens."""
        return self.motion_state != 0

    @property
    def humidity_sensor_state(self):
        """Return state of humidity event state."""
        # NA(0) or Disarmed (1) or Normal(2) or TooDry(3) or TooHumid(4) or ThresholdPending
        return self._info['capEventState']

    @property
    def is_humidity_sensor_armed(self):
        """Return True if tag armed and listens to humidity changes events."""
        cap_state = self.humidity_sensor_state
        return cap_state not in [0, 1]

    @property
    def moisture_sensor_state(self):
        """Return moisture state."""
        return self.humidity_sensor_state

    @property
    def is_moisture_sensor_armed(self):
        """Return True if tag armed and listens to moisture level events."""
        return self.is_humidity_sensor_armed

    @property
    def temperature_sensor_state(self):
        """Return state of temp sensor monitoring."""
        # Disarmed(0) or Normal(1) or TooHigh(2) or TooLow(3) or ThresholdPending(4)
        return self._info['tempEventState']

    @property
    def is_temperature_sensor_armed(self):
        """Return True if tag armed and listens to temperature changes evens."""
        return self.temperature_sensor_state != 0

    @property
    def light_sensor_state(self):
        """Return state of light sensor monitoring.

        States: NA (0) or Disarmed(1) or Normal or TooDark or TooBright or ThresholdPending.
        """
        return self._info['lightEventState']

    @property
    def is_light_sensor_armed(self):
        """Return True if tag armed and listens to light changes evens."""
        light_state = self.light_sensor_state
        return light_state not in [0, 1]

    @property
    def is_moved(self):
        """Return True if detected movement."""
        return self.motion_state == 2  # Moved

    @property
    def is_door_open(self):
        """Return True if detected door opening."""
        return self.motion_state == 3  # Open

    @property
    def is_cold(self):
        """Return True if temperature is too low."""
        return self.temperature_sensor_state == 3  # TooLow

    @property
    def is_heat(self):
        """Return True if temperature is too high."""
        return self.temperature_sensor_state == 2  # TooHigh

    @property
    def is_too_dry(self):
        """Return True if humidity is too low (<20%)."""
        return self.humidity_sensor_state == 3  # TooDry

    @property
    def is_too_humid(self):
        """Return True if humidity is too wet (>80%)."""
        return self.humidity_sensor_state == 4  # TooHumid

    @property
    def is_light_on(self):
        """Return True if detected light."""
        return self.light > 0

    @property
    def is_leaking(self):
        """Return True if detected water leak - applicable for water sensor only."""
        return self.tag_type == CONST.WIRELESSTAG_TYPE_WATER and self.moisture > 0

    @property
    def is_battery_low(self):
        """Return True if detected low battery level."""
        return self.battery_volts <= self.low_battery_threshold

    @property
    def supported_binary_events_types(self):
        """Return supported by tag binary event types (events with state on or off)."""
        events_map = {
            # 13-bit tag - allows everything but not light and moisture
            CONST.WIRELESSTAG_TYPE_13BIT: [
                CONST.EVENT_PRESENCE, CONST.EVENT_BATTERY,
                CONST.EVENT_MOTION, CONST.EVENT_DOOR,
                CONST.EVENT_COLD, CONST.EVENT_HEAT,
                CONST.EVENT_DRY, CONST.EVENT_WET],

            # Moister/water sensor - temperature and moisture only
            CONST.WIRELESSTAG_TYPE_WATER: [
                CONST.EVENT_PRESENCE, CONST.EVENT_BATTERY,
                CONST.EVENT_COLD, CONST.EVENT_HEAT,
                CONST.EVENT_MOISTURE],

            # ALS Pro: allows everything, but not moisture
            CONST.WIRELESSTAG_TYPE_ALSPRO: [
                CONST.EVENT_PRESENCE, CONST.EVENT_BATTERY,
                CONST.EVENT_MOTION, CONST.EVENT_DOOR,
                CONST.EVENT_COLD, CONST.EVENT_HEAT,
                CONST.EVENT_DRY, CONST.EVENT_WET,
                CONST.EVENT_LIGHT],

            # PIR KumoSensor: specialized on motion, but has
            # temperature and humidity as 13-bit tag
            CONST.WIRELESSTAG_TYPE_PIR: [
                CONST.EVENT_PRESENCE, CONST.EVENT_BATTERY,
                CONST.EVENT_MOTION, CONST.EVENT_DOOR,
                CONST.EVENT_COLD, CONST.EVENT_HEAT,
                CONST.EVENT_DRY, CONST.EVENT_WET
            ],

            # Wemo are power switches.
            CONST.WIRELESSTAG_TYPE_WEMO_DEVICE: [CONST.EVENT_PRESENCE]
        }

        # allow everything if tag type is unknown
        # (i just dont have full catalog of them :))
        tag_type = self.tag_type
        fullset = BINARY_EVENT_SPECS.keys()
        return events_map[tag_type] if tag_type in events_map else fullset

    def event_for_type(self, event_type):
        """Return event model for specified type."""
        if event_type not in self.supported_binary_events_types:
            return None

        if event_type in self._event_specs:
            return self._event_specs[event_type]
        else:
            event = BinaryEvent.make_event(event_type, self)
            self._event_specs[event_type] = event
            return event

    @property
    def allowed_sensor_types(self):
        """Return array of allowed sensor types for tag."""
        all_sensors = [CONST.SENSOR_TEMPERATURE,
                       CONST.SENSOR_HUMIDITY,
                       CONST.SENSOR_LIGHT]
        sensors_per_tag_type = {
            CONST.WIRELESSTAG_TYPE_13BIT: [
                CONST.SENSOR_TEMPERATURE,
                CONST.SENSOR_HUMIDITY],
            CONST.WIRELESSTAG_TYPE_WATER: [
                CONST.SENSOR_TEMPERATURE,
                CONST.SENSOR_MOISTURE],
            CONST.WIRELESSTAG_TYPE_ALSPRO: [
                CONST.SENSOR_TEMPERATURE,
                CONST.SENSOR_HUMIDITY,
                CONST.SENSOR_LIGHT],
            CONST.WIRELESSTAG_TYPE_PIR: [
                CONST.SENSOR_TEMPERATURE,
                CONST.SENSOR_HUMIDITY
            ],
            CONST.WIRELESSTAG_TYPE_WEMO_DEVICE: []
        }

        tag_type = self.tag_type
        return (
            sensors_per_tag_type[tag_type] if tag_type in sensors_per_tag_type
            else all_sensors)

    def sensor_for_type(self, sensor_type):
        """Return sensor for specified type or None if not supported by tag."""
        if sensor_type not in self.allowed_sensor_types:
            return None

        if sensor_type in self._sensors_spec:
            return self._sensors_spec[sensor_type]
        else:
            sensor = Sensor.make_sensor(sensor_type, self)
            self._sensors_spec[sensor_type] = sensor
            return sensor

    @property
    def allowed_monitoring_types(self):
        """Return allowed actions to monitor for tag."""
        all_sensors = [
            CONST.ARM_TEMPERATURE, CONST.ARM_HUMIDITY,
            CONST.ARM_MOTION, CONST.ARM_LIGHT]

        sensors_per_tag_spec = {
            CONST.WIRELESSTAG_TYPE_13BIT: [
                CONST.ARM_TEMPERATURE, CONST.ARM_HUMIDITY, CONST.ARM_MOTION],
            CONST.WIRELESSTAG_TYPE_WATER: [
                CONST.ARM_TEMPERATURE, CONST.ARM_MOISTURE],
            CONST.WIRELESSTAG_TYPE_ALSPRO: [
                CONST.ARM_TEMPERATURE, CONST.ARM_HUMIDITY,
                CONST.ARM_MOTION, CONST.ARM_LIGHT],
            CONST.WIRELESSTAG_TYPE_PIR: [
                CONST.ARM_TEMPERATURE, CONST.ARM_HUMIDITY, CONST.ARM_MOTION
            ],
            CONST.WIRELESSTAG_TYPE_WEMO_DEVICE: []
        }

        tag_type = self.tag_type

        result = (
            sensors_per_tag_spec[tag_type]
            if tag_type in sensors_per_tag_spec else all_sensors)
        _LOGGER.info("Allowed switches: %s tag_type: %s",
                     str(result), tag_type)

        return result

    def __getattr__(self, key):
        """Return proxy models for subscripting support."""
        if key == 'sensor':
            return SensorProxy(self)
        elif key == 'event':
            return BinaryEventProxy(self)
        else:
            raise AttributeError

    def __repr__(self):
        """Return string representation of tag."""
        # Water/Moisture Sensor supports water and temperature
        if self.tag_type == CONST.WIRELESSTAG_TYPE_WATER:
            return '{} temp: {} leak: {}'.format(self.name, self.temperature, self.is_leaking)
        # ALS Pro (8bit)
        elif self.tag_type == CONST.WIRELESSTAG_TYPE_ALSPRO:
            return '{} temp: {} humidity: {} lux: {}'.format(self.name, self.temperature, self.humidity, self.light)

        # use 13-bit tag supports temp/motion/humidity as fallback for everything else
        return '{} temp: {} humidity: {}'.format(self.name, self.temperature, self.humidity)



#!/usr/bin/python
# -*- coding: utf-8 -*-

"""
Model class representing sensor of specific type.

Single tag contains multiple number of sensors such as
temperature, motion, moisture, humidity or light.

Copyrights: (c) 2018 Sergiy Maysak, see LICENSE file for details
Creation Date: 8/30/2018
"""

import logging
import weakref
import wirelesstagpy.constants as CONST

_LOGGER = logging.getLogger(__name__)

SENSOR_TYPES = {
    CONST.SENSOR_TEMPERATURE: {
        'unit': '°C',
        'attr': 'temperature',
        'update_attr': 'temp'
    },
    CONST.SENSOR_HUMIDITY: {
        'unit': '%',
        'attr': 'humidity',
        'update_attr': 'cap'
    },
    CONST.SENSOR_MOISTURE: {
        'unit': '%',
        'attr': 'moisture',
        'update_attr': 'cap'
    },
    CONST.SENSOR_LIGHT: {
        'unit': 'lux',
        'attr': 'light',
        'update_attr': 'lux'
    }
}


class Sensor:
    """Entity representing single sensor."""

    @classmethod
    def make_sensor(cls, sensor_type, tag):
        """Build sensor for type with parent tag."""
        spec = SENSOR_TYPES[sensor_type]
        return cls(sensor_type, spec, tag)

    def __init__(self, sensor_type, spec, tag):
        """Init with spec and parent tag."""
        self.sensor_type = sensor_type
        self.unit = spec['unit']
        self._attr = spec['attr']
        self._update_attr = spec['update_attr']
        self._parent_tag = weakref.ref(tag)

    @property
    def value(self):
        """Return current value of sensor."""
        return getattr(self._parent_tag(), self._attr, False)

    def value_from_update_event(self, event_spec):
        """Decode new value from arrived update event."""
        try:
            return event_spec[self._update_attr]
        except Exception as error:
            _LOGGER.error("Failed to get value from event data: %s - %s",
                          event_spec, error)
            return 0

    def __repr__(self):
        """Return string representation."""
        return '{} value: {} of {}'.format(self.sensor_type, self.value, self.unit)


class SensorProxy:
    """Proxy model to handle subscripting for single sensor."""

    def __init__(self, tag):
        """Init proxy with parent tag."""
        self._parent_tag = weakref.ref(tag)

    def __getitem__(self, key):
        """Return sensor instance for key."""
        return self._parent_tag().sensor_for_type(key)



#!/usr/bin/python
# -*- coding: utf-8 -*-

"""
Utilities functions for wirelesstagpy module.

Copyrights: (c) 2018 Sergiy Maysak, see LICENSE file for details
Creation Date: 3/8/2018.
"""

from datetime import timedelta, datetime

EPOCH_AS_FILETIME = 116444736000000000


def convert_filetime_to_dt(file_time):
    """Convert filetime to datetime."""
    u_microseconds = (file_time - EPOCH_AS_FILETIME) // 10
    return datetime(1970, 1, 1) + timedelta(microseconds=u_microseconds)



#!/usr/bin/python
# -*- coding: utf-8 -*-

"""WirelessTags exceptions."""


class WirelessTagsException(Exception):
    """WirelessTags exception."""

    def __init__(self, message):
        """Initialize WirelessTagsException."""
        super(WirelessTagsException, self).__init__(message)
        self.message = message



#!/usr/bin/python
# -*- coding: utf-8 -*-

"""WirelessTags constants."""

MAJOR_VERSION = 0
MINOR_VERSION = 4
PATCH_VERSION = 1

__version__ = '{}.{}.{}'.format(MAJOR_VERSION, MINOR_VERSION, PATCH_VERSION)

HEADERS = {"content-type": "application/json; charset=utf-8"}
BASEURL = "https://my.wirelesstag.net"

SIGN_IN_URL = BASEURL + "/ethAccount.asmx/SignInEx"
IS_SIGNED_IN_URL = BASEURL + "/ethAccount.asmx/IsSignedInEx"
GET_TAGS_URL = BASEURL + "/ethClient.asmx/GetTagList2"

ARM_MOTION_URL = BASEURL + "/ethClient.asmx/Arm"
ARM_HUMIDITY_URL = BASEURL + "/ethClient.asmx/ArmCapSensor"
ARM_TEMPERATURE_URL = BASEURL + "/ethClient.asmx/ArmTempSensor"
ARM_LIGHT_URL = BASEURL + "/ethClient.asmx/ArmLightSensor"

DISARM_MOTION_URL = BASEURL + "/ethClient.asmx/DisArm"
DISARM_HUMIDITY_URL = BASEURL + "/ethClient.asmx/DisarmCapSensor"
DISARM_TEMPERATURE_URL = BASEURL + "/ethClient.asmx/DisarmTempSensor"
DISARM_LIGHT_URL = BASEURL + "/ethClient.asmx/DisarmLightSensor"

LOAD_EVENT_URL_CONFIG_URL = BASEURL + "/ethClient.asmx/LoadEventURLConfig"
SAVE_EVENT_URL_CONFIG_URL = BASEURL + "/ethClient.asmx/SaveEventURLConfig"

WIRELESSTAG_TYPE_13BIT = 13
WIRELESSTAG_TYPE_ALSPRO = 26
WIRELESSTAG_TYPE_WATER = 32
WIRELESSTAG_TYPE_PIR = 72
WIRELESSTAG_TYPE_WEMO_DEVICE = 82

# events
# On means in range, Off means out of range
EVENT_PRESENCE = 'presence'

# On means motion detected, Off means cear
EVENT_MOTION = 'motion'

# On means open, Off means closed
EVENT_DOOR = 'door'

# On means temperature become too cold, Off means normal
EVENT_COLD = 'cold'

# On means hot, Off means normal
EVENT_HEAT = 'heat'

# On means too dry (humidity), Off means normal
EVENT_DRY = 'dry'

# On means too wet (humidity), Off means normal
EVENT_WET = 'wet'

# On means light detected, Off means no light
EVENT_LIGHT = 'light'

# On means moisture detected (wet), Off means no moisture (dry)
EVENT_MOISTURE = 'moisture'

# On means tag battery is low, Off means normal
EVENT_BATTERY = 'battery'

# supported sensor types
SENSOR_TEMPERATURE = 'temperature'
SENSOR_HUMIDITY = 'humidity'
SENSOR_MOISTURE = 'moisture'
SENSOR_LIGHT = 'light'

# supported actions to monitoring specific metric
ARM_TEMPERATURE = 'temperature'
ARM_HUMIDITY = 'humidity'
ARM_MOTION = 'motion'
ARM_LIGHT = 'light'
ARM_MOISTURE = 'moisture'



#!/usr/bin/env python3
"""wirelesstagpy setup script."""

from __future__ import print_function
import io

from setuptools import setup, find_packages

import wirelesstagpy.constants as CONST

# read the contents of your README file
from os import path
this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

PACKAGES = find_packages(exclude=['test', 'test.*', 'test*'])

setup(
    name='wirelesstagpy',
    version=CONST.__version__,
    url='https://github.com/sergeymaysak/wirelesstagpy/',
    license='MIT',
    author='Sergiy Maysak',
    tests_require=['pytest'],
    install_requires=['requests>=2.18.4'],
    author_email='sergey.maysak@gmail.com',
    description='Simple python wrapper over wirelesstags REST API',
    long_description=long_description,
    long_description_content_type='text/markdown',
    packages=PACKAGES,
    python_requires='>=3',
    py_modules=['wirelesstagpy'],
    include_package_data=True,
    platforms='any',
    test_suite='test',
    classifiers=[
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Development Status :: 4 - Beta',
        'Natural Language :: English',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Topic :: Home Automation'
        ],
    extras_require={
        'testing': ['unittest'],
    }
)



#!/usr/bin/python
# -*- coding: utf-8 -*-

"""
Model class for push notification config for wirelesstags platform.

Copyrights: (c) 2018 Sergiy Maysak, see LICENSE file for details
Creation Date: 3/20/2018
"""


class NotificationConfig:
    """Model representing push notification configuration for single tag."""

    @classmethod
    def make_post_local(cls, name, url, content):
        """Create local push notification using POST http verb."""
        return cls(name, {'url': url, 'verb': 'POST',
                          'content': content,
                          'disabled': False, 'nat': True})

    @classmethod
    def make_config_for_update_event(cls, url, mac):
        """Create local push notification for update sensor data."""
        content = ("{\"name\":\"{0}\",\"id\":{1},\"temp\":{2}," +
                   "\"cap\":{3},\"lux\":{4},\"mac\":\"" + mac +
                   "\"}")
        return cls.make_post_local('update', url, content)

    def __init__(self, name, spec):
        """Init with name of event and dictionary."""
        self.name = name
        self.spec = spec

    @property
    def url(self):
        """URL to receive notification."""
        return self.spec["url"]

    @property
    def verb(self):
        """HTTP verb: GET, POST etc."""
        return self.spec["verb"]

    @property
    def is_enabled(self):
        """Is notification enabled or not."""
        return not self.spec["disabled"]

    @property
    def is_local(self):
        """Is notification local."""
        return self.spec["nat"]

    @property
    def content(self):
        """Notification tempplate to be sent."""
        return self.spec["content"]

    def __repr__(self):
        """Return string representation."""
        return '{} spec: {}'.format(self.name, self.spec)
